"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const tf = __importStar(require("@tensorflow/tfjs-node-gpu"));
const assert = __importStar(require("assert"));
const path = __importStar(require("path"));
const fs = __importStar(require("fs"));
const jimp_1 = __importDefault(require("jimp"));
/** @ignore
 * assertion test
 * @param data
 */
const assertionTest = (data) => {
    assert.ok(data.metadata.feature, 'Image feature is missing');
    assert.ok(data.metadata.feature.shape.length === 3, 'The size of an image must be 3d');
};
function argMax(array) {
    return [].map.call(array, (x, i) => [x, i]).reduce((r, a) => (a[0] > r[0] ? a : r))[1];
}
/**
 * this is the plugin used to load a simple cnn model or load existing model.
 * @param optimizer (string | tf.train.Optimizer)[optional / default = tf.train.rmsprop(0.00005, 1e-7)] the optimizer of model
 * @param loss (string | string [] | {[outputName: string]: string} | LossOrMetricFn | LossOrMetricFn [] | {[outputName: string]: LossOrMetricFn}) \
 * [optional / default = 'categoricalCrossentropy'] the loss function of model
 * @param metrics (string | LossOrMetricFn | Array | {[outputName: string]: string | LossOrMetricFn}): [optional / default = ['accuracy']] \
 * the evaluation metrics of model during training
 * @param modelId (string)[optional] if you want to load a model from previously trained pipcook pipeline, give the pipeline id here
 * @param modelPath (string)[optional] if you want to load a model from a local file path, give the path here
 */
const simpleCnnModelDefine = (data, args) => __awaiter(void 0, void 0, void 0, function* () {
    let { optimizer = tf.train.rmsprop(0.00005, 1e-7), loss = 'categoricalCrossentropy', metrics = ['accuracy'], recoverPath, outputShape, labelMap } = args;
    let inputShape;
    // create a new model
    if (!recoverPath) {
        assertionTest(data);
        inputShape = data.metadata.feature.shape;
        outputShape = Object.keys(data.metadata.labelMap).length;
        labelMap = data.metadata.labelMap;
    }
    else {
        const log = JSON.parse(fs.readFileSync(path.join(recoverPath, '..', 'metadata.json'), 'utf8'));
        labelMap = JSON.parse(log.output.dataset).labelMap;
        outputShape = Object.keys(labelMap).length;
    }
    let model = null;
    // load from former pipcook trained model
    if (recoverPath) {
        model = (yield tf.loadLayersModel('file://' + path.join(recoverPath, 'model.json')));
    }
    else {
        const localModel = tf.sequential();
        localModel.add(tf.layers.conv2d({
            inputShape: inputShape,
            filters: 32,
            kernelSize: 3,
            activation: 'relu',
            kernelInitializer: 'glorotUniform',
            padding: 'same'
        }));
        localModel.add(tf.layers.conv2d({
            filters: 32,
            kernelSize: 3,
            activation: 'relu',
            kernelInitializer: 'glorotUniform'
        }));
        localModel.add(tf.layers.maxPooling2d({ poolSize: [2, 2] }));
        localModel.add(tf.layers.dropout({ rate: 0.25 }));
        localModel.add(tf.layers.conv2d({
            filters: 64,
            kernelSize: 3,
            activation: 'relu',
            kernelInitializer: 'glorotUniform',
            padding: 'same'
        }));
        localModel.add(tf.layers.conv2d({
            filters: 64,
            kernelSize: 3,
            activation: 'relu',
            kernelInitializer: 'glorotUniform'
        }));
        localModel.add(tf.layers.maxPooling2d({ poolSize: [2, 2] }));
        localModel.add(tf.layers.dropout({ rate: 0.25 }));
        localModel.add(tf.layers.flatten());
        localModel.add(tf.layers.dense({ units: 512, activation: 'relu', kernelInitializer: 'glorotUniform' }));
        localModel.add(tf.layers.dropout({ rate: 0.5 }));
        localModel.add(tf.layers.dense({ units: outputShape, activation: 'softmax', kernelInitializer: 'glorotUniform' }));
        model = localModel;
    }
    model.compile({
        optimizer,
        loss,
        metrics
    });
    return {
        model,
        metrics,
        predict: function (inputData) {
            return __awaiter(this, void 0, void 0, function* () {
                let predict;
                const image = yield jimp_1.default.read(inputData.data);
                const trainImageBuffer = yield image.getBufferAsync(jimp_1.default.MIME_JPEG);
                const imageArray = new Uint8Array(trainImageBuffer);
                const imgTensor = tf.node.decodeImage(imageArray, 3);
                const predictResultArray = this.model.predict(imgTensor.expandDims(0));
                const index = argMax(predictResultArray.dataSync());
                if (labelMap) {
                    for (let key in labelMap) {
                        if (labelMap[key] === index) {
                            predict = key;
                        }
                    }
                }
                else {
                    predict = predictResultArray;
                }
                return predict;
            });
        }
    };
});
exports.default = simpleCnnModelDefine;
//# sourceMappingURL=index.js.map